(:  ======================================================================  :)
(:  =====      DRINKING WATER DIRECIVE SCHEMA VALIDATION                       =====  :)
(:  ======================================================================  :)
(:
    This XQuery contains a number of validations of dwd reports to
    help MS evaluate data. The validations are for informtion only and
    will not prevent data from being stored in the database.

    XQuery agency:  The European Commission
    XQuery version: 1.0
    XQuery date:    
    Revision:       $Revision: 1 $
    Revision date:  $Date: 2011-10-10 $
  :)

xquery version "1.0";

(:===================================================================:)
(: Namespace declaration                                             :)
(:===================================================================:)

import module namespace xmlutil = "urn:eu:com:env:dwd:converter:standard:util:1" at "http://icm.eionet.europa.eu/schemas/dir199883ec/xquery/DWD_xquery_utility.xquery";

declare namespace xmlconv="urn:eu:com:env:dwd:converter:standard:1";
declare namespace dwd="http://icm.eionet.europa.eu/schemas/dir199883ec";


declare namespace functx = "http://www.functx.com"; 
declare function functx:path-to-node 
  ( $nodes as node()* )  as xs:string* {
    $nodes/string-join(ancestor-or-self::*/name(.), '/')
 } ;

(:===================================================================:)
(: Code lists:)
(:===================================================================:)

(:===================================================================:)
(: Variable given as an external parameter by the QA service         :)
(:===================================================================:)
declare variable $source_url as xs:string external; 


(:==================================================================:)
(: *******   VALIDATIONS ******************************** :)
(:==================================================================:)
  
declare function xmlconv:numberSensePopn($source_url){
for $i in doc($source_url)/dwd:DWD/dwd:Table1_GeneralInformation[dwd:TotalPopulation > 100]
													
       return xmlutil:buildValidationReturn("TotalPopulation", functx:path-to-node($i), (data($i/dwd:TotalPopulation)), 1, "1","" , $i)
						
};


declare function xmlconv:numberSenseVolume($source_url){
for $i in doc($source_url)/dwd:DWD/dwd:Table1_GeneralInformation[dwd:TotalVolumeSuppliedWater < 10]
													
       return xmlutil:buildValidationReturn("TotalVolumeSuppliedWater", functx:path-to-node($i), (data($i/dwd:TotalVolumeSuppliedWater)), 1, "2", "", $i)
						
};

declare function xmlconv:percentageCheck($source_url){
for $i in doc($source_url)/dwd:DWD/dwd:Table1_GeneralInformation
let $sum:=sum(($i/dwd:PercentGroundwater,$i/dwd:PercentSurfaceWater,$i/dwd:PercentCoastalTransitionalWater,$i/dwd:PercentInlandWater,$i/dwd:PercentBankFiltrationWater,$i/dwd:PercentArtificialGroundWaterRecharge,$i/dwd:PercentRainWater,$i/dwd:PercentOtherSources))
where $sum != 100													
       return xmlutil:buildValidationReturn("Percent values", functx:path-to-node($i), xs:string($sum), 1, "3", "", $i)
						
};

declare function xmlconv:WSZcheck1($source_url) {	
	for $i in doc($source_url)/dwd:DWD/dwd:Table1_GeneralInformation/dwd:WaterSupplyZones/dwd:WaterSupplyZone
        let $ii := doc($source_url)/dwd:DWD/dwd:Table1_GeneralInformation/dwd:WaterSupplyZones/dwd:WaterSupplyZone[dwd:WSZ_ID = $i/dwd:WSZ_ID]	
        where count($ii) > 1
        return xmlutil:buildValidationReturn("WSZ_ID", functx:path-to-node($i), data($i/dwd:WSZ_ID), 1, "6", "", "") 
};

declare function xmlconv:WSZcheck2($source_url) {	
	
    for $i in doc($source_url)/dwd:DWD/dwd:Table4_AnnualMonitoring/dwd:AnnualMonitoring
        let $ii := doc($source_url)/dwd:DWD/dwd:Table1_GeneralInformation/dwd:WaterSupplyZones/dwd:WaterSupplyZone[dwd:WSZ_ID = $i/dwd:WSZ_ID]	
        where count($ii) < 1
        return xmlutil:buildValidationReturn("WSZ_ID", functx:path-to-node($i), data($i/dwd:WSZ_ID), 1, "5", "", "") 
			
};

declare function xmlconv:WSZcheck3($source_url){
let $MStotal := doc($source_url)/dwd:DWD/dwd:Table1_GeneralInformation/dwd:NoWaterSupplyZones
for $NStotal in doc($source_url)/dwd:DWD/dwd:Table5A_NationalSummary/dwd:Parameters
where xs:double($NStotal/dwd:WSZ_Monitored) > xs:double($MStotal)               												
       return xmlutil:buildValidationReturn("WSZ_Monitored", functx:path-to-node($NStotal/dwd:WSZ_Monitored), concat("MS: ",$MStotal," - NS: ",$NStotal/dwd:WSZ_Monitored), 1, "4", data($NStotal/dwd:Parameter), $NStotal)
						
};

declare function xmlconv:WSZcheck4($source_url) {	
	
    for $i in doc($source_url)/dwd:DWD/dwd:Table6_NonComplianceInformation/dwd:Parameters
        let $ii := doc($source_url)/dwd:DWD/dwd:Table1_GeneralInformation/dwd:WaterSupplyZones/dwd:WaterSupplyZone[dwd:WSZ_ID = $i/dwd:WSZ_ID]	
        where count($ii) < 1
        return xmlutil:buildValidationReturn("WSZ_ID", functx:path-to-node($i), data($i/dwd:WSZ_ID), 1, "5", "", "") 
			
};


(:LocationInformation must be populated if YesNo = "Y" in table 7 :)
declare function xmlconv:MissingLocationInfo($source_url){

for $i in doc($source_url)/dwd:DWD/dwd:Table7_QualityInformation/dwd:row/*[dwd:YesNo = 'Y' and count(dwd:LocationofInformation) < 1]


 return 
	xmlutil:buildValidationReturn("LocationInformation", functx:path-to-node($i), data($i/dwd:YesNo), 1, "7", "", "")
};

(:LocationInformation must be populated if YesNo = "Y" in table 8 :)
declare function xmlconv:MissingLocationInfo8($source_url){

for $i in doc($source_url)/dwd:DWD/dwd:Table8_PublicInformation/dwd:row/*[dwd:YesNo = 'Y' and count(dwd:LocationofInformation) < 1]


 return 
	xmlutil:buildValidationReturn("LocationInformation", functx:path-to-node($i), data($i/dwd:YesNo), 1, "7", "", "")
};
(:==================================================================:)
(: :)
(:==================================================================:)
declare function xmlconv:conditionalValidation($source_url as xs:string){

let $numberSensePopn := xmlconv:numberSensePopn($source_url)
let $percentageCheck := xmlconv:percentageCheck($source_url)
let $WSZcheck1 := xmlconv:WSZcheck1($source_url)
let $WSZcheck2 := xmlconv:WSZcheck2($source_url)
let $WSZcheck3 := xmlconv:WSZcheck3($source_url)
let $WSZcheck4 := xmlconv:WSZcheck4($source_url)
let $MissingLocationInfo := xmlconv:MissingLocationInfo($source_url)
let $MissingLocationInfo8 := xmlconv:MissingLocationInfo8($source_url)

let $hasErrors :=  exists($numberSensePopn) 	or
							exists($percentageCheck) or
							exists($WSZcheck1) or
							exists($WSZcheck2) or
							exists($WSZcheck3)  or
							exists($WSZcheck4) or
							exists($MissingLocationInfo) or
							exists($MissingLocationInfo8)
return
		if( $hasErrors) then
		(
			<div>
				{
                    xmlutil:buildTableErrorReport($numberSensePopn union $percentageCheck union $WSZcheck1 union $WSZcheck2 union $WSZcheck3 union $WSZcheck4 union $MissingLocationInfo union $MissingLocationInfo8)
				}
			</div>
		)
		else
		(
			<div>
				{xmlutil:buildDescription("All validation checks were passed successfully.")}
			</div>
		 )
};

(:==================================================================:)
(: *******  RUN VALIDATIONS AND GENERATE OUTPUT *************************************** :)
(:==================================================================:)

declare function xmlconv:complexValidation($source_url as xs:string){

<div>
	
		<h3>1. Conditional checks</h3>
		
				{xmlconv:conditionalValidation($source_url)}
			
</div>	
};

(:==================================================================:)
(: Main function:)
(:==================================================================:)
declare function xmlconv:proceed($source_url as xs:string) {
<div class="feedbacktext">
    <h1>Drinking Water Directive validation</h1>
    
    <h2>Complex validation checks</h2>
    <br/>
    {xmlconv:complexValidation($source_url)}

</div>
};

xmlconv:proceed($source_url)
